# A simple script to use with a Statestep model treated as a
# decision table, in which some variables are treated as inputs
# and the others (if any) as outputs.
# A suitable model with which to try this is activitiesOutcomes
# (one of the sample models supplied with Statestep): in that
# model, the inputs are 'Temperature', 'Weather' and 'Wind' and
# there is one output variable: 'Activity'.
# If you have generated activitiesOutcomes.pm, do:
#
#   perl -MactivitiesOutcomes=:all decision.pl

BEGIN {
    die 'No Statestep model (run "perl -M<model>=:all decision.pl")'
        unless defined $Combination;
    die 'Model contains no variables with values' unless @Vars;
    die 'Model contains no events' unless @Events;
}

use 5.008008;
use warnings;
use strict;

# map variable names to numbers
my %var_num = map { $Vars[$_] => $_ } 0 .. $#Vars;

decision_table();

sub decision_table {
    my @inputs = get_input_vars();
    my @outputs;
    for my $var (0 .. $#Vars) {
        push @outputs, $var unless grep { $var == $_ } @inputs;
    }
    my $comb = $Combination->new();
    print "Below, enter '.' at any time to quit.\n";
    for (;;) {
        print "Input:\n", map
            " $Vars[$_]=$Values[$_][$comb->value($_)]\n", @inputs;
        print "Enter 'variable=value' assignments to ",
            "change the input (optional):\n";
        my $in = get_input();
        last if $in eq '.';
        my @args = $in =~ /(\w+)\s*=\s*(\w+)/g;
        eval { $comb = $comb->new(@args); };
        print $@ if $@;
        if ($@ || !@args && $in) {
            print "The input variables and possible values are:\n";
            print " $Vars[$_]: ", join(', ', @{$Values[$_]}), ".\n"
                for @inputs;
            next;
        }
        print "Looking at rules for event '$Events[0]' ...\n";
        if (my ($rule, $comb2) = $comb->apply_rules(0)) {
            print "Rule '$Rules[0][$rule]' applies.\n";
            if (@outputs) {
                print "Output:\n", map
                    " $Vars[$_]=$Values[$_][$comb2->value($_)]\n",
                    @outputs;
            }
        }
        else {
            print "No rule applies for this input.\n";
        }
        print "\n";
    }
}

# find which variables are the decision table's inputs
sub get_input_vars {
    my @inputs;
    until (@inputs) {
        print 'The variables are: ', join(', ', @Vars), ".\n";
        print "Which of these are input variables?\n";
        my $in;
        $in = get_input() while not $in;
        for my $var ($in =~ /\w+/g) {
            my $num = $var_num{"$var"};
            if (defined $num) {
                push @inputs, $num;
            }
            else {
                print("'$var' is not a variable.\n");
                @inputs = ();
                last;
            }
        }
    }
    return sort @inputs;
}

# get a line from standard input
sub get_input {
    print '> ';
    my $in = <>;
    defined $in or print("\n"), exit 0;
    print $in unless -t;
    $in =~ s/^\s+//;
    $in =~ s/\s+$//;
    $in;
}
