#!/usr/bin/env perl

use 5.008008;
use warnings;
use strict;

use File::Compare qw/compare_text/;

# directory containing files used for testing
my $test_dir = 'in/';
# contains files derived from test files or created during testing
my $temp_dir = 'gen/';
# directory containing template sample .pl files to test
my $samples_dir = '../';

# The XPath library or libraries used to generate the
# Perl code from the template files.
my @xpathlibs;
open my $makefile, 'makefile' or die;
while (<$makefile>) {
    s/^xpathlibs\W*// and @xpathlibs = split and last;
}
@xpathlibs or die "No xpathlibs found in makefile";


xpathlib_equiv();
standard();
test_sample_progs();
test_extralogic();
test_formulas();


# Where Perl code has been generated using more than one
# XPath library, check that the code generated by each is
# equivalent. (Provided it is, it then suffices to test
# the code generated using any one of them.)
sub xpathlib_equiv {
    @xpathlibs > 1 or return;
    for my $x (0 .. $#xpathlibs) {
        my $xlib = $xpathlibs[$x];
        for my $file (glob(
            "$temp_dir?*_$xlib*.p[ml]"))
        {
            for my $y (0 .. $#xpathlibs) {
                next if $y == $x;
                my $ylib = $xpathlibs[$y];
                (my $other = $file) =~ s/_$xlib/_$ylib/;
                die "FAILED: file '$file' but no file '$other'"
                    unless -e $other;
                if ($x > $y) {
                    die "FAILED: '$file' and '$other' differ "
                        . "(after allowing for package naming)"
                    if compare_text($file, $other,
                        sub {
                            $_[0] =~ s/_$xlib/_$ylib/;
                            $_[0] ne $_[1];
                        });
                }
            }
        }
    }
    print 'Files generated for libs ', join(', ', @xpathlibs),
        " are equivalent.\n";
}

# tests for models test*.s2, big*.s2
# test code generated with name_dirs set
sub standard {
    my @tests = glob("{$temp_dir}*.exp");
    die "FAILED - no standard tests found" unless @tests;
    push @tests, glob("${test_dir}big*.exp");
    my $fail = 0;
    for my $test (@tests) {
        my $exp = $test;
        $test =~ s/.exp$//;
        $test =~ s/^$test_dir/$temp_dir/;
        # for these tests, the package name includes the subdir
        (my $package = $test) =~ s#/#::#;
        $package .= "_$xpathlibs[0]_model";
        my $out = $test . '.out';
        $test =~ s#.*/##;
        print "Test $test ";
        system("perl -M$package=:all ./test_model.pl >$out 2>&1");
        print compare_text($out, $exp)
            ? $fail = "FAILED: $out and $exp differ.\n"
            : "passed.\n";
    }
    die "FAILED one or more tests (see above)" if $fail;
}

# test fsm.pl, decision.pl
# test code generated with name_dirs unset
# (i.e., no directories in package name)
sub test_sample_progs {
    my $pattern = join ' ', map $test_dir . $_. '_*.exp',
        qw/fsm decision/;
    my $fail;
    for my $exp (glob($pattern)) {
        (my $test = $exp) =~ s/$test_dir//;
        $test =~ s/.exp$//;
        my ($prog, $pack) = split /_/, $test;
        $prog .= '.pl';
        $pack = $pack . "_$xpathlibs[0]_model";
        my $out = "$temp_dir$test.out";
        system "sed -n 's/^> \\?//p' $exp|perl -I$temp_dir " .
            "-M$pack=:all $samples_dir$prog >$out 2>&1\n";
        print "Test $test ", compare_text($out, $exp)
            ? my $fail = "FAILED: $out and $exp differ.\n"
            : "passed.\n"
    }
    die "FAILED one or more tests (see above)" if $fail;
}

sub test_extralogic {
    print "Test extralogic.pl.ftl ";
    system("perl -I$temp_dir -Mextra_$xpathlibs[0]_model=:all "
        . "test_extralogic.pl extra_$xpathlibs[0]_extralogic.pl")
        and die "FAILED" or print "passed.\n";
}

sub test_formulas {
    print "Test formula text extraction ";
    system("perl -I$temp_dir -Mformulas_$xpathlibs[0]_model=:all "
        . "test_formulas.pl")
        and die "FAILED" or print "passed.\n";
}
